@@ -21,6 +21,7 @@
 #ifndef LILV_LILV_H
 #define LILV_LILV_H
 
+#include <stdarg.h>
 #include <stddef.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -464,6 +465,13 @@ LILV_API
 bool
 lilv_nodes_contains(const LilvNodes* values, const LilvNode* value);
 
+/**
+   Return a new LilvNodes that contains all nodes from both @p a and @p b.
+*/
+LILV_API
+LilvNodes*
+lilv_nodes_merge(const LilvNodes* a, const LilvNodes* b);
+
 /* Plugins */
 
 LILV_API
@@ -641,6 +649,38 @@ lilv_world_find_nodes(LilvWorld*      world,
                       const LilvNode* predicate,
                       const LilvNode* object);
 
+/**
+   Find a single node that matches a pattern.
+   Exactly one of @p subject, @p predicate, @p object must be NULL.
+   This function is equivalent to
+   lilv_nodes_get_first(lilv_world_find_nodes(...)) but simplifies the common
+   case of only wanting a single value.
+   @return the first matching node, or NULL if no matches are found.
+*/
+LILV_API
+LilvNode*
+lilv_world_get(LilvWorld*      world,
+               const LilvNode* subject,
+               const LilvNode* predicate,
+               const LilvNode* object);
+
+/**
+   Return true iff a statement matching a certain pattern exists.
+
+   This is useful for checking if particular statement exists without having to
+   bother with collections and memory management.
+
+   @param subject Subject of statement, or NULL for anything.
+   @param predicate Predicate (key) of statement, or NULL for anything.
+   @param object Object (value) of statement, or NULL for anything.
+*/
+LILV_API
+bool
+lilv_world_ask(LilvWorld*      world,
+               const LilvNode* subject,
+               const LilvNode* predicate,
+               const LilvNode* object);
+
 /**
    @}
    @name Plugin
@@ -858,6 +898,19 @@ uint32_t
 lilv_plugin_get_num_ports_of_class(const LilvPlugin* p,
                                    const LilvNode*   class_1, ...);
 
+#ifndef SWIG
+/**
+   Variant of lilv_plugin_get_num_ports_of_class() that takes a va_list.
+
+   This function calls va_arg() on @p args but does not call va_end().
+*/
+LILV_API
+uint32_t
+lilv_plugin_get_num_ports_of_class_va(const LilvPlugin* p,
+                                      const LilvNode*   class_1,
+                                      va_list           args);
+#endif
+
 /**
    Return whether or not the plugin introduces (and reports) latency.
    The index of the latency port can be found with lilv_plugin_get_latency_port
@@ -908,7 +961,7 @@ lilv_plugin_get_port_by_symbol(const LilvPlugin* plugin,
    ports for a particular designation.
 */
 LILV_API
-LilvPort*
+const LilvPort*
 lilv_plugin_get_port_by_designation(const LilvPlugin* plugin,
                                     const LilvNode*   port_class,
                                     const LilvNode*   designation);
@@ -1019,6 +1072,19 @@ lilv_port_get_value(const LilvPlugin* plugin,
                     const LilvPort*   port,
                     const LilvNode*   predicate);
 
+/**
+   Get a single property value of a port.
+
+   This is equivalent to lilv_nodes_get_first(lilv_port_get_value(...)) but is
+   simpler to use in the common case of only caring about one value.  The
+   caller is responsible for freeing the returned node.
+*/
+LILV_API
+LilvNode*
+lilv_port_get(const LilvPlugin* plugin,
+              const LilvPort*   port,
+              const LilvNode*   predicate);
+
 /**
    Return the LV2 port properties of a port.
 */
@@ -1037,13 +1103,16 @@ lilv_port_has_property(const LilvPlugin* p,
                        const LilvNode*   property_uri);
 
 /**
-   Return whether a port is an event port and supports a certain event type.
+   Return whether a port supports a certain event type.
+
+   More precisely, this returns true iff the port has an atom:supports or an
+   ev:supportsEvent property with @p event_type as the value.
 */
 LILV_API
 bool
 lilv_port_supports_event(const LilvPlugin* p,
                          const LilvPort*   port,
-                         const LilvNode*   event_uri);
+                         const LilvNode*   event_type);
 
 /**
    Get the index of a port.
@@ -1138,6 +1207,8 @@ lilv_port_get_scale_points(const LilvPlugin* plugin,
 
 /**
    Load a state snapshot from @c world's RDF model.
+   This function can be used to load the default state of a plugin by passing
+   the plugin URI as the @p subject parameter.
    @param subject The subject of the state description (e.g. a preset URI).
    @return A new LilvState which must be freed with lilv_state_free().
 */
@@ -1377,6 +1448,11 @@ lilv_state_save(LilvWorld*                 world,
 
 /**
    Save state to a string.  This function does not use the filesystem.
+
+   @param uri URI for the state description (mandatory).
+   @param base_uri Base URI for serialisation.  Unless you know what you are
+   doing, pass NULL for this, otherwise the state may not be restorable via
+   lilv_state_new_from_string().
 */
 LILV_API
 char*
