@@ -136,6 +136,7 @@ struct ScalePoints {
 struct Nodes {
 	LILV_WRAP_COLL(Nodes, Node, nodes);
 	LILV_WRAP1(bool, nodes, contains, const Node, node);
+	LILV_WRAP0(Node, nodes, get_first);
 };
 
 struct Port {
@@ -190,6 +191,9 @@ struct Plugin {
 	LILV_WRAP0(Node,        plugin, get_author_email);
 	LILV_WRAP0(Node,        plugin, get_author_homepage);
 	LILV_WRAP0(bool,        plugin, is_replaced);
+	LILV_WRAP0(Nodes,       plugin, get_extension_data);
+	LILV_WRAP0(UIs,         plugin, get_uis);
+	LILV_WRAP1(Nodes,       plugin, get_related, Node, type);
 
 	inline Port get_port_by_index(unsigned index) {
 		return Port(me, lilv_plugin_get_port_by_index(me, index));
@@ -206,10 +210,15 @@ struct Plugin {
 			me, min_values, max_values, def_values);
 	}
 
-	inline unsigned get_num_ports_of_class(LilvNode* class_1,
-	                                       LilvNode* class_2) {
-		// TODO: varargs
-		return lilv_plugin_get_num_ports_of_class(me, class_1, class_2, NULL);
+	inline unsigned get_num_ports_of_class(LilvNode* class_1, ...) {
+		va_list args;
+		va_start(args, class_1);
+
+		const uint32_t count = lilv_plugin_get_num_ports_of_class_va(
+			me, class_1, args);
+
+		va_end(args);
+		return count;
 	}
 
 	const LilvPlugin* me;
@@ -299,6 +308,7 @@ struct World {
 	LILV_WRAP0(const LilvPluginClass*, world, get_plugin_class);
 	LILV_WRAP0(const LilvPluginClasses*, world, get_plugin_classes);
 	LILV_WRAP0(const Plugins, world, get_all_plugins);
+	LILV_WRAP1(int, world, load_resource, const LilvNode*, resource);
 
 	LilvWorld* me;
 };
