@@ -432,15 +432,6 @@ lilv_state_restore(const LilvState*           state,
 	}
 }
 
-static SordNode*
-get1(SordModel* model, const SordNode* s, const SordNode* p)
-{
-	SordIter* const i    = sord_search(model, s, p, 0, 0);
-	SordNode* const node = i ? sord_node_copy(sord_iter_get_node(i, SORD_OBJECT)) : NULL;
-	sord_iter_free(i);
-	return node;
-}
-
 static LilvState*
 new_state_from_model(LilvWorld*       world,
                      LV2_URID_Map*    map,
@@ -463,6 +454,12 @@ new_state_from_model(LilvWorld*       world,
 			state->dir = lilv_strdup((const char*)sord_node_get_string(graph));
 		}
 		sord_iter_free(i);
+	} else if (sord_ask(model,
+	                    node,
+	                    world->uris.rdf_a,
+	                    world->uris.lv2_Plugin, 0)) {
+		// Loading plugin description as state (default state)
+		state->plugin_uri = lilv_node_new_from_node(world, node);
 	} else {
 		LILV_ERRORF("State %s missing lv2:appliesTo property\n",
 		            sord_node_get_string(node));
@@ -490,18 +487,18 @@ new_state_from_model(LilvWorld*       world,
 	// Get port values
 	SordIter* ports = sord_search(model, node, world->uris.lv2_port, 0, 0);
 	FOREACH_MATCH(ports) {
-		const SordNode* port   = sord_iter_get_node(ports, SORD_OBJECT);
-		const SordNode* label  = get1(model, port, world->uris.rdfs_label);
-		const SordNode* symbol = get1(model, port, world->uris.lv2_symbol);
-		const SordNode* value  = get1(model, port, world->uris.pset_value);
+		const SordNode* port = sord_iter_get_node(ports, SORD_OBJECT);
+
+		SordNode* label  = sord_get(model, port, world->uris.rdfs_label, 0, 0);
+		SordNode* symbol = sord_get(model, port, world->uris.lv2_symbol, 0, 0);
+		SordNode* value  = sord_get(model, port, world->uris.pset_value, 0, 0);
+		if (!value) {
+			value = sord_get(model, port, world->uris.lv2_default, 0, 0);
+		}
 		if (!symbol) {
 			LILV_ERRORF("State `%s' port missing symbol.\n",
 			            sord_node_get_string(node));
-		} else if (!value) {
-			LILV_ERRORF("State `%s' port `%s' missing value.\n",
-			            sord_node_get_string(symbol),
-			            sord_node_get_string(node));
-		} else {
+		} else if (value) {
 			chunk.len = 0;
 			sratom_read(sratom, &forge, world->world, model, value);
 			LV2_Atom* atom = (LV2_Atom*)chunk.buf;
@@ -515,12 +512,15 @@ new_state_from_model(LilvWorld*       world,
 				                     (const char*)sord_node_get_string(label));
 			}
 		}
+		sord_node_free(world->world, value);
+		sord_node_free(world->world, symbol);
+		sord_node_free(world->world, label);
 	}
 	sord_iter_free(ports);
 
 	// Get properties
-	SordNode* statep = sord_new_uri(world->world, USTR(LV2_STATE__state));
-	const SordNode* state_node = get1(model, node, statep);
+	SordNode* statep     = sord_new_uri(world->world, USTR(LV2_STATE__state));
+	SordNode* state_node = sord_get(model, node, statep, NULL, NULL);
 	if (state_node) {
 		SordIter* props = sord_search(model, state_node, 0, 0, 0);
 		FOREACH_MATCH(props) {
@@ -553,6 +553,7 @@ new_state_from_model(LilvWorld*       world,
 		}
 		sord_iter_free(props);
 	}
+	sord_node_free(world->world, state_node);
 	sord_node_free(world->world, statep);
 
 	free((void*)chunk.buf);
@@ -642,6 +643,10 @@ lilv_state_new_from_string(LilvWorld*    world,
                            LV2_URID_Map* map,
                            const char*   str)
 {
+	if (!str) {
+		return NULL;
+	}
+
 	SerdNode    base   = SERD_NODE_NULL;
 	SerdEnv*    env    = serd_env_new(&base);
 	SordModel*  model  = sord_new(world->world, SORD_SPO|SORD_OPS, false);
@@ -650,14 +655,13 @@ lilv_state_new_from_string(LilvWorld*    world,
 	set_prefixes(env);
 	serd_reader_read_string(reader, USTR(str));
 
-	const SordNode* p = sord_new_uri(world->world, USTR(LILV_NS_RDF "type"));
-	const SordNode* o = sord_new_uri(world->world, USTR(LV2_PRESETS__Preset));
-	SordIter* const i = sord_search(model, NULL, p, o, NULL);
-	const SordNode* s = sord_iter_get_node(i, SORD_SUBJECT);
+	SordNode* o = sord_new_uri(world->world, USTR(LV2_PRESETS__Preset));
+	SordNode* s = sord_get(model, NULL, world->uris.rdf_a, o, NULL);
 
 	LilvState* state = new_state_from_model(world, map, model, s, NULL);
 
-	sord_iter_free(i);
+	sord_node_free(world->world, s);
+	sord_node_free(world->world, o);
 	serd_reader_free(reader);
 	sord_free(model);
 	serd_env_free(env);
@@ -984,6 +988,11 @@ lilv_state_to_string(LilvWorld*       world,
                      const char*      uri,
                      const char*      base_uri)
 {
+	if (!uri) {
+		LILV_ERROR("Attempt to serialise state with no URI\n");
+		return NULL;
+	}
+
 	SerdChunk   chunk  = { NULL, 0 };
 	SerdEnv*    env    = NULL;
 	SerdNode    base   = serd_node_from_string(SERD_URI, USTR(base_uri));
