@@ -49,6 +49,10 @@
 #    include <sys/file.h>
 #endif
 
+#ifndef PAGE_SIZE
+#   define PAGE_SIZE 4096
+#endif
+
 char*
 lilv_strjoin(const char* first, ...)
 {
@@ -65,7 +69,12 @@ lilv_strjoin(const char* first, ...)
 			break;
 
 		const size_t this_len = strlen(s);
-		result = (char*)realloc(result, len + this_len + 1);
+		if (!(result = (char*)realloc(result, len + this_len + 1))) {
+			free(result);
+			LILV_ERROR("realloc() failed\n");
+			return NULL;
+		}
+
 		memcpy(result + len, s, this_len);
 		len += this_len;
 	}
@@ -133,7 +142,7 @@ lilv_get_lang(void)
 }
 
 /** Append suffix to dst, update dst_len, and return the realloc'd result. */
-char*
+static char*
 strappend(char* dst, size_t* dst_len, const char* suffix, size_t suffix_len)
 {
 	dst = (char*)realloc(dst, *dst_len + suffix_len + 1);
@@ -143,7 +152,7 @@ strappend(char* dst, size_t* dst_len, const char* suffix, size_t suffix_len)
 }
 
 /** Append the value of the environment variable var to dst. */
-char*
+static char*
 append_var(char* dst, size_t* dst_len, const char* var)
 {
 	// Get value from environment
@@ -269,9 +278,8 @@ lilv_copy_file(const char* src, const char* dst)
 		return 2;
 	}
 
-	static const size_t PAGE_SIZE = 4096;
-	char*               page      = (char*)malloc(PAGE_SIZE);
-	size_t              n_read    = 0;
+	char*  page   = (char*)malloc(PAGE_SIZE);
+	size_t n_read = 0;
 	while ((n_read = fread(page, 1, PAGE_SIZE, in)) > 0) {
 		if (fwrite(page, 1, n_read, out) != n_read) {
 			LILV_ERRORF("write to %s failed (%s)\n", dst, strerror(errno));
