@@ -185,31 +185,37 @@ lilv_world_find_nodes(LilvWorld*      world,
 		LILV_ERRORF("Subject `%s' is not a resource\n",
 		            sord_node_get_string(subject->node));
 		return NULL;
-	}
-	if (!lilv_node_is_uri(predicate)) {
+	} else if (!lilv_node_is_uri(predicate)) {
 		LILV_ERRORF("Predicate `%s' is not a URI\n",
 		            sord_node_get_string(predicate->node));
 		return NULL;
-	}
-	if (!subject && !object) {
+	} else if (!subject && !object) {
 		LILV_ERROR("Both subject and object are NULL\n");
 		return NULL;
 	}
+	
+	return lilv_world_find_nodes_internal(
+		world,
+		subject ? subject->node : NULL,
+		predicate->node,
+		object ? object->node : NULL);
+}
 
-	SordNode* const subject_node = subject
-		? sord_node_copy(subject->node)
-		: NULL;
-
-	SordNode* const object_node = object
-		? sord_node_copy(object->node)
-		: NULL;
-
-	LilvNodes* ret = lilv_world_query_values_internal(
-		world, subject_node, predicate->node, object_node);
-
-	sord_node_free(world->world, subject_node);
-	sord_node_free(world->world, object_node);
-	return ret;
+LILV_API
+LilvNode*
+lilv_world_get(LilvWorld*      world,
+               const LilvNode* subject,
+               const LilvNode* predicate,
+               const LilvNode* object)
+{
+	SordNode* snode = sord_get(world->model,
+	                           subject   ? subject->node   : NULL,
+	                           predicate ? predicate->node : NULL,
+	                           object    ? object->node    : NULL,
+	                           NULL);
+	LilvNode* lnode = lilv_node_new_from_node(world, snode);
+	sord_node_free(world->world, snode);
+	return lnode;
 }
 
 SordIter*
@@ -221,11 +227,31 @@ lilv_world_query_internal(LilvWorld*      world,
 	return sord_search(world->model, subject, predicate, object, NULL);
 }
 
+bool
+lilv_world_ask_internal(LilvWorld*      world,
+                        const SordNode* subject,
+                        const SordNode* predicate,
+                        const SordNode* object)
+{
+	return sord_ask(world->model, subject, predicate, object, NULL);
+}
+
+LILV_API
+bool
+lilv_world_ask(LilvWorld*      world,
+               const LilvNode* subject,
+               const LilvNode* predicate,
+               const LilvNode* object)
+{
+	return sord_ask(
+		world->model, subject->node, predicate->node, object->node, NULL);
+}
+
 LilvNodes*
-lilv_world_query_values_internal(LilvWorld*      world,
-                                 const SordNode* subject,
-                                 const SordNode* predicate,
-                                 const SordNode* object)
+lilv_world_find_nodes_internal(LilvWorld*      world,
+                               const SordNode* subject,
+                               const SordNode* predicate,
+                               const SordNode* object)
 {
 	return lilv_nodes_from_stream_objects(
 		world,
@@ -667,49 +693,28 @@ lilv_world_load_plugin_classes(LilvWorld* world)
 	FOREACH_MATCH(classes) {
 		const SordNode* class_node = sord_iter_get_node(classes, SORD_SUBJECT);
 
-		// Get parents (superclasses)
-		SordIter* parents = sord_search(
-			world->model,
-			class_node,
-			world->uris.rdfs_subClassOf,
-			NULL,
-			NULL);
-
-		if (sord_iter_end(parents)) {
-			sord_iter_free(parents);
+		SordNode* parent = sord_get(
+			world->model, class_node, world->uris.rdfs_subClassOf, NULL, NULL);
+		if (!parent || sord_node_get_type(parent) != SORD_URI) {
 			continue;
 		}
 
-		const SordNode* parent_node = sord_iter_get_node(parents, SORD_OBJECT);
-		sord_iter_free(parents);
-
-		if (!sord_node_get_type(parent_node) == SORD_URI) {
-			// Class parent is not a resource, ignore (e.g. owl restriction)
+		SordNode* label = sord_get(
+			world->model, class_node, world->uris.rdfs_label, NULL, NULL);
+		if (!label) {
+			sord_node_free(world->world, parent);
 			continue;
 		}
 
-		// Get labels
-		SordIter* labels = sord_search(
-			world->model,
-			class_node,
-			world->uris.rdfs_label,
-			NULL,
-			NULL);
-
-		if (sord_iter_end(labels)) {
-			sord_iter_free(labels);
-			continue;
-		}
-
-		const SordNode* label_node = sord_iter_get_node(labels, SORD_OBJECT);
-		const uint8_t*  label      = sord_node_get_string(label_node);
-		sord_iter_free(labels);
-
 		LilvPluginClass* pclass = lilv_plugin_class_new(
-			world, parent_node, class_node, (const char*)label);
+			world, parent, class_node,
+			(const char*)sord_node_get_string(label));
 		if (pclass) {
 			zix_tree_insert((ZixTree*)world->plugin_classes, pclass, NULL);
 		}
+
+		sord_node_free(world->world, label);
+		sord_node_free(world->world, parent);
 	}
 	sord_iter_free(classes);
 }
@@ -730,19 +735,14 @@ lilv_world_load_all(LilvWorld* world)
 			(ZixTree*)world->plugins, p);
 
 		// ?new dc:replaces plugin
-		SordIter* replacement = sord_search(
-			world->model,
-			NULL,
-			world->uris.dc_replaces,
-			lilv_node_as_node(lilv_plugin_get_uri(plugin)),
-			NULL);
-		if (!sord_iter_end(replacement)) {
-			/* TODO: Check if replacement is actually a known plugin,
-			   though this is expensive...
-			*/
+		if (sord_ask(world->model,
+		             NULL,
+		             world->uris.dc_replaces,
+		             lilv_node_as_node(lilv_plugin_get_uri(plugin)),
+		             NULL)) {
+			// TODO: Check if replacement is a known plugin? (expensive)
 			((LilvPlugin*)plugin)->replaced = true;
 		}
-		sord_iter_free(replacement);
 	}
 
 	// Query out things to cache
